/*!
 * @asigloo/vue-dynamic-forms v1.x.x
 * (c) 2020 Alvaro Saburido
 * @license MIT
 */
var asVueDynamicForms = (function (exports, vue) {
  'use strict';

  const dynamicFormsSymbol = Symbol();
  function useDynamicForms() {
      const dynamicForms = vue.inject(dynamicFormsSymbol);
      if (!dynamicForms)
          throw new Error('No dynamicForms provided!!!');
      return dynamicForms;
  }

  /* eslint-disable */
  function useInputEvents(props, emit) {
      function onChange($event) {
          if (props.control) {
              props.control.dirty = true;
          }
      }
      function onCheck($event) {
          if (props.control) {
              props.control.dirty = true;
          }
      }
      function onFocus() {
          emit('focus');
      }
      function onBlur() {
          emit('blur');
          if (props.control) {
              props.control.touched = true;
          }
      }
      vue.watch(props, (form) => {
          var _a;
          if ((form === null || form === void 0 ? void 0 : form.control) && !((_a = form === null || form === void 0 ? void 0 : form.control) === null || _a === void 0 ? void 0 : _a.dirty)) {
              form.control.dirty = true;
              emit('change', form.control.value);
          }
      });
      return {
          onFocus,
          onChange,
          onBlur,
          onCheck,
      };
  }

  const props = {
      control: Object,
  };
  var script = vue.defineComponent({
      name: 'asTextInput',
      props,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
          return () => {
              var _a;
              return vue.h('input', {
                  id: props.control.name,
                  name: props.control.name || '',
                  type: props.control.type,
                  class: ['form-control'],
                  value: (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.value,
                  disabled: props.control.disabled,
                  placeholder: props.control.placeholder,
                  required: props.control.required,
                  readonly: props === null || props === void 0 ? void 0 : props.control.readonly,
                  autocomplete: props.control.autocomplete,
                  ariaRequired: props.control.required,
                  ariaLabel: props.control.ariaLabel,
                  ariaLabelledBy: props.control.ariaLabelledBy,
                  onFocus,
                  onBlur,
                  onChange,
              });
          };
      },
  });

  const render = () => {};


  script.render = render;
  script.__file = "src/components/text-input/TextInput.vue";

  const props$1 = {
      control: Object,
  };
  var script$1 = vue.defineComponent({
      name: 'asSelectInput',
      props: props$1,
      setup(props, { emit }) {
          return () => {
              var _a, _b, _c, _d, _e, _f;
              const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
              const options = (_b = (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map(({ key, value, disabled }) => vue.h('option', { key, value: key, disabled }, value));
              return vue.h('select', {
                  id: props.control.name,
                  name: ((_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.name) || '',
                  class: ['form-control'],
                  value: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.value,
                  disabled: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.disabled,
                  placeholder: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.placeholder,
                  required: props.control.required,
                  readonly: props === null || props === void 0 ? void 0 : props.control.readonly,
                  ariaLabel: props.control.ariaLabel,
                  ariaLabelledBy: props.control.ariaLabelledBy,
                  ariaRequired: props.control.required,
                  onFocus,
                  onBlur,
                  onChange,
              }, options);
          };
      },
  });

  const render$1 = () => {};


  script$1.render = render$1;
  script$1.__file = "src/components/select-input/SelectInput.vue";

  const props$2 = {
      control: Object,
  };
  var script$2 = vue.defineComponent({
      name: 'asTextAreaInput',
      props: props$2,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
          return () => {
              var _a, _b, _c, _d, _e, _f;
              return vue.h('textarea', {
                  id: props.control.name,
                  name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                  class: ['form-control'],
                  value: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.value,
                  rows: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.rows,
                  cols: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.cols,
                  disabled: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.disabled,
                  placeholder: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.placeholder,
                  required: props.control.required,
                  autocomplete: props.control.autocomplete,
                  readonly: props === null || props === void 0 ? void 0 : props.control.readonly,
                  ariaLabel: props.control.ariaLabel,
                  ariaLabelledBy: props.control.ariaLabelledBy,
                  ariaRequired: props.control.required,
                  onFocus,
                  onBlur,
                  onChange,
              });
          };
      },
  });

  const render$2 = () => {};


  script$2.render = render$2;
  script$2.__file = "src/components/text-area-input/TextAreaInput.vue";

  const props$3 = {
      control: Object,
  };
  var script$3 = vue.defineComponent({
      name: 'asCheckboxInput',
      props: props$3,
      setup(props, { emit }) {
          var _a, _b, _c, _d, _e, _f, _g, _h;
          const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
          const renderCheckbox = [
              vue.h('input', {
                  name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                  type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                  id: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.name,
                  disabled: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.disabled,
                  class: ['checkbox-control'],
                  value: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.value,
                  checked: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.value,
                  onFocus,
                  onBlur,
                  onChange,
              }),
              vue.h('label', {
                  class: ['checkbox-label'],
                  for: (_g = props === null || props === void 0 ? void 0 : props.control) === null || _g === void 0 ? void 0 : _g.name,
              }, (_h = props === null || props === void 0 ? void 0 : props.control) === null || _h === void 0 ? void 0 : _h.label),
          ];
          return () => [
              vue.h('div', {
                  class: 'checkbox-group',
                  tabIndex: -1,
                  role: 'group',
              }, renderCheckbox),
          ];
      },
  });

  const render$3 = () => {};


  script$3.render = render$3;
  script$3.__file = "src/components/checkbox-input/CheckboxInput.vue";

  const props$4 = {
      control: Object,
  };
  var script$4 = vue.defineComponent({
      name: 'asRadioInput',
      props: props$4,
      setup(props, { emit }) {
          var _a, _b;
          const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
          const renderRadios = (_b = (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map(option => {
              var _a, _b;
              return vue.h('div', { class: 'radio-input' }, [
                  vue.h('input', {
                      name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                      type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                      id: option.key,
                      disabled: option.disabled,
                      class: ['radio-control'],
                      value: option.value,
                      onFocus,
                      onBlur,
                      onChange,
                  }),
                  vue.h('label', {
                      class: ['radio-label'],
                      for: option.key,
                  }, option.value),
              ]);
          });
          return () => [
              vue.h('div', {
                  class: 'radio-group',
                  tabIndex: -1,
                  role: 'group',
              }, renderRadios),
          ];
      },
  });

  const render$4 = () => {};


  script$4.render = render$4;
  script$4.__file = "src/components/radio-input/RadioInput.vue";

  const props$5 = {
      control: Object,
  };
  var script$5 = vue.defineComponent({
      name: 'asNumberInput',
      props: props$5,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props, emit);
          return () => {
              var _a, _b, _c, _d, _e, _f, _g, _h;
              return vue.h('input', {
                  id: props.control.name,
                  name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                  type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                  class: ['form-control'],
                  value: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.value,
                  min: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.min,
                  max: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.max,
                  step: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.step,
                  disabled: (_g = props === null || props === void 0 ? void 0 : props.control) === null || _g === void 0 ? void 0 : _g.disabled,
                  placeholder: (_h = props === null || props === void 0 ? void 0 : props.control) === null || _h === void 0 ? void 0 : _h.placeholder,
                  required: props.control.required,
                  readonly: props === null || props === void 0 ? void 0 : props.control.readonly,
                  autocomplete: props.control.autocomplete,
                  ariaLabel: props.control.ariaLabel,
                  ariaLabelledBy: props.control.ariaLabelledBy,
                  ariaRequired: props.control.required,
                  onFocus,
                  onBlur,
                  onChange,
              });
          };
      },
  });

  const render$5 = () => {};


  script$5.render = render$5;
  script$5.__file = "src/components/number-input/NumberInput.vue";

  /* eslint-disable */
  const { assign, entries, values, keys } = Object;
  const isArray = (a) => !!a && a.constructor === Array;
  const isObject = (a) => !!a && a.constructor === Object;
  const isEvent = (e) => !!e && e.constructor === Event;
  const isEmpty = (entry) => {
      if (isArray(entry)) {
          return entry.length > 0;
      }
      if (isObject(entry)) {
          return entries(entry).length > 0;
      }
  };
  const removeEmpty = obj => Object.keys(obj)
      .filter(k => obj[k] != null) // Remove undef. and null.
      .reduce((newObj, k) => typeof obj[k] === 'object'
      ? { ...newObj, [k]: removeEmpty(obj[k]) } // Recurse.
      : { ...newObj, [k]: obj[k] }, // Copy value.
  {});

  /* eslint-disable @typescript-eslint/no-use-before-define */
  const components = {
      TextInputComponent: script,
      SelectInputComponent: script$1,
      TextAreaInputComponent: script$2,
      CheckboxInputComponent: script$3,
      RadioInputComponent: script$4,
      NumberInputComponent: script$5,
  };
  const props$6 = {
      control: {
          type: Object,
          required: true,
      },
      submited: {
          type: Boolean,
          required: true,
      },
  };
  var script$6 = vue.defineComponent({
      name: 'asDynamicInput',
      components,
      props: props$6,
      setup(props, { emit, slots }) {
          const { onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          const { options } = vue.inject(dynamicFormsSymbol);
          let component;
          const attributes = vue.computed(() => {
              return {
                  control: props === null || props === void 0 ? void 0 : props.control,
                  style: props === null || props === void 0 ? void 0 : props.control.customStyles,
                  onChange: valueChange,
              };
          });
          const hasLabel = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) !== 'checkbox'; });
          const isFieldSet = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === 'radio'; });
          const getClasses = vue.computed(() => {
              var _a, _b, _c, _d, _e, _f;
              const classes = [
                  'dynamic-input',
                  'form-group',
                  {
                      'form-group--inline': ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === "checkbox" /* CHECKBOX */,
                  },
                  {
                      'form-group--error': showErrors.value,
                  },
              ];
              if (isArray((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.customClass)) {
                  return [
                      ...classes,
                      ...(_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.customClass,
                  ];
              }
              if (isObject((_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.customClass)) {
                  return [...classes, (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.customClass];
              }
              return [classes, (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.customClass];
          });
          const autoValidate = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.touched) && (options === null || options === void 0 ? void 0 : options.autoValidate); });
          const showErrors = vue.computed(() => {
              var _a, _b;
              return (((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.errors) &&
                  keys((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.errors).length > 0 &&
                  (props.submited || autoValidate.value));
              /* props.control.errors &&
                Object.keys(props.control.errors).length > 0 &&
                (this.submited || this.autoValidate) */
          });
          const errorMessages = vue.computed(() => {
              var _a;
              const errors = values(((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.errors) || {});
              if (errors.length > 0 && (props.submited || autoValidate.value)) {
                  return errors.map(value => value.text);
              }
              return [];
          });
          function validate() {
              if (props.control &&
                  props.control.validations &&
                  isEmpty(props.control.validations)) {
                  const validation = props.control.validations.reduce((prev, curr) => {
                      const val = typeof curr.validator === 'function'
                          ? curr.validator(props.control)
                          : null;
                      if (val !== null) {
                          const [key, value] = entries(val)[0];
                          const obj = {};
                          obj[key] = {
                              value,
                              text: curr.text,
                          };
                          return {
                              ...prev,
                              ...obj,
                          };
                      }
                      return {
                          ...prev,
                      };
                  }, {});
                  props.control.errors = validation;
                  props.control.valid = Object.keys(validation).length === 0;
              }
          }
          function valueChange($event) {
              let value;
              const newValue = {};
              if (isEvent($event)) {
                  $event.stopPropagation();
                  value =
                      props.control.type === 'checkbox'
                          ? $event.target.checked
                          : $event.target.value;
              }
              else {
                  value = $event;
              }
              if (props.control) {
                  newValue[props.control.name] = value;
                  validate();
                  emit('change', newValue);
              }
          }
          return () => {
              var _a, _b, _c, _d;
              switch ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) {
                  case "text" /* TEXT */:
                      component = vue.h(script, attributes.value);
                      break;
                  case 'email':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'password':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'url':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'color':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'number':
                      component = vue.h(script$5, attributes.value);
                      break;
                  case 'select':
                      component = vue.h(script$1, attributes.value);
                      break;
                  case 'textarea':
                      component = vue.h(script$2, attributes.value);
                      break;
                  case 'checkbox':
                      component = vue.h(script$3, attributes.value);
                      break;
                  case 'radio':
                      component = vue.h(script$4, attributes.value);
                      break;
                  case 'custom-field':
                      component = vue.h('slot', {
                          name: 'customField',
                      }, slots.customField({
                          control: props.control,
                          onChange: valueChange,
                          onFocus,
                          onBlur,
                      }));
                      break;
              }
              const requiredStar = vue.h('span', {
                  ariaHidden: true,
                  class: 'form-required-star',
              }, ' *');
              return vue.h(isFieldSet.value ? 'fieldset' : 'div', {
                  class: getClasses.value,
                  role: isFieldSet.value ? undefined : 'group',
              }, [
                  hasLabel.value
                      ? vue.h(isFieldSet.value ? 'legend' : 'label', {
                          class: 'form-label',
                          for: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.name,
                      }, [
                          `${(_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.label}`,
                          ((_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.required) ? requiredStar : '',
                      ])
                      : null,
                  component,
                  vue.h('div', {
                      class: 'form-errors',
                  }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
              ]);
          };
      },
  });

  const render$6 = () => {};


  script$6.render = render$6;
  script$6.__file = "src/components/dynamic-input/DynamicInput.vue";

  /* import { warn } from '../../core/utils/warning';
   */
  const props$7 = {
      form: {
          type: Object,
          required: true,
      },
  };
  const components$1 = {
      DynamicInput: script$6,
  };
  const EMPTY_CONTROL = {
      dirty: false,
      touched: false,
      valid: true,
  };
  /* const AVAILABLE_THEMES = ['default', 'material'];
   */
  var script$7 = vue.defineComponent({
      name: 'asDynamicForm',
      props: props$7,
      components: components$1,
      setup(props, ctx) {
          const { options } = vue.inject(dynamicFormsSymbol);
          const controls = vue.ref([]);
          const formValues = vue.reactive({});
          const submited = vue.ref(false);
          vue.onMounted(() => {
              mapControls();
              initValues();
          });
          //  TODO: enable again when plugin theme option is available
          /* const validTheme = computed(
            () => options.theme && AVAILABLE_THEMES.includes(options.theme),
          );
      
          if (!validTheme.value) {
            warn(
              `There isn't a theme: ${
                options.theme
              } just yet, please choose one of the available themes: ${AVAILABLE_THEMES.join(
                ', ',
              )}`,
            );
          } */
          const deNormalizedScopedSlots = vue.computed(() => Object.keys(ctx.slots));
          const normalizedControls = vue.computed(() => {
              let normalizedControls = {};
              controls.value.forEach(element => {
                  normalizedControls[element.name] = element;
              });
              return normalizedControls;
          });
          const isValid = vue.computed(() => {
              const hasInvalidControls = controls.value.some(control => !control.valid);
              return !hasInvalidControls;
          });
          const errors = vue.computed(() => {
              return controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const errors = Object.keys(curr.errors || {}) || [];
                      if (errors.length > 0) {
                          const error = {};
                          error[curr.name] = curr.errors;
                          return {
                              ...prev,
                              ...error,
                          };
                      }
                      return prev;
                  }, {})
                  : {};
          });
          const formattedOptions = vue.computed(() => {
              if (options === null || options === void 0 ? void 0 : options.form) {
                  const { customClass, customStyles, method, netlify, netlifyHoneypot, autocomplete, } = options === null || options === void 0 ? void 0 : options.form;
                  return {
                      class: customClass,
                      style: customStyles,
                      method,
                      'data-netlify': netlify,
                      'data-netlify-honeypot': netlifyHoneypot,
                      autocomplete: autocomplete ? 'on' : 'off',
                  };
              }
              else {
                  return;
              }
          });
          function valueChange(changedValue) {
              Object.assign(formValues, changedValue);
              ctx.emit('change', removeEmpty(formValues));
          }
          function mapControls(empty) {
              var _a;
              const controlArray = Object.entries((_a = props.form) === null || _a === void 0 ? void 0 : _a.fields).map(([key, field]) => empty
                  ? {
                      ...field,
                      name: key,
                      value: null,
                      ...EMPTY_CONTROL,
                  }
                  : {
                      ...field,
                      name: key,
                      ...EMPTY_CONTROL,
                  }) || [];
              if (props.form.fieldOrder) {
                  controls.value = controlArray.sort((a, b) => props.form.fieldOrder.indexOf(a.name) -
                      props.form.fieldOrder.indexOf(b.name));
              }
              else {
                  controls.value = controlArray;
              }
          }
          function resetForm() {
              mapControls(true);
          }
          function handleSubmit() {
              submited.value = true;
              if (isValid.value) {
                  ctx.emit('submited', formValues);
                  resetForm();
              }
              else {
                  ctx.emit('error', formValues);
              }
          }
          function initValues() {
              Object.assign(formValues, controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const obj = {};
                      obj[curr.name] =
                          curr.type === "number" /* NUMBER */
                              ? parseFloat(`${curr.value}`)
                              : curr.value;
                      return {
                          ...prev,
                          ...obj,
                      };
                  }, {})
                  : {});
              ctx.emit('changed', formValues);
          }
          vue.watch(props.form.fields, () => {
              mapControls();
          });
          return {
              controls,
              form: props.form,
              valueChange,
              formValues,
              handleSubmit,
              isValid,
              errors,
              deNormalizedScopedSlots,
              normalizedControls,
              submited,
              formattedOptions,
          };
      },
  });

  function render$7(_ctx, _cache, $props, $setup, $data, $options) {
    const _component_dynamic_input = vue.resolveComponent("dynamic-input");

    return (vue.openBlock(), vue.createBlock("form", vue.mergeProps({
      class: "dynamic-form",
      novalidate: "",
      id: _ctx.form.id,
      name: _ctx.form.id
    }, _ctx.formattedOptions, {
      onSubmit: _cache[1] || (_cache[1] = vue.withModifiers((...args) => (_ctx.handleSubmit(...args)), ["prevent"]))
    }), [
      (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.controls, (control) => {
        return (vue.openBlock(), vue.createBlock(_component_dynamic_input, {
          key: control.name,
          control: control,
          submited: _ctx.submited,
          onChange: _ctx.valueChange
        }, {
          customField: vue.withCtx((props) => [
            (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.deNormalizedScopedSlots, (slot) => {
              return (vue.openBlock(), vue.createBlock("div", {
                key: slot,
                class: "custom-form-wrapper"
              }, [
                (props.control.name === slot)
                  ? vue.renderSlot(_ctx.$slots, slot, {
                      key: 0,
                      control: _ctx.normalizedControls[slot],
                      onChange: props.onChange,
                      onFocus: props.onFocus,
                      onBlur: props.onBlur
                    })
                  : vue.createCommentVNode("v-if", true)
              ]))
            }), 128 /* KEYED_FRAGMENT */))
          ]),
          _: 2
        }, 1032 /* PROPS, DYNAMIC_SLOTS */, ["control", "submited", "onChange"]))
      }), 128 /* KEYED_FRAGMENT */))
    ], 16 /* FULL_PROPS */, ["id", "name"]))
  }

  script$7.render = render$7;
  script$7.__file = "src/components/dynamic-form/DynamicForm.vue";

  function createDynamicForms(options) {
      const vdf = {
          options,
          install(app) {
              const self = this;
              app.component('dynamic-form', script$7);
              app.provide(dynamicFormsSymbol, self);
          },
      };
      return vdf;
  }

  const FieldBase = ({ validations = [], label = null, ariaLabel = null, ariaLabelledBy = null, customClass = null, customStyles = null, disabled = false, placeholder = null, required = false, autocomplete = null, readonly = false, }) => ({
      validations,
      label,
      ariaLabel,
      ariaLabelledBy,
      customClass,
      customStyles,
      disabled,
      placeholder,
      required,
      autocomplete,
      readonly,
  });
  const TextField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "text" /* TEXT */,
  });
  const EmailField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "email" /* EMAIL */,
  });
  const PasswordField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "password" /* PASSWORD */,
  });
  const CheckboxField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "checkbox" /* CHECKBOX */,
  });
  const ColorField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "color" /* COLOR */,
  });
  const RadioField = ({ options, value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      options,
      type: "radio" /* RADIO */,
  });
  const NumberField = ({ value = null, min = 0, max = 100, step = 1, ...rest }) => ({
      ...FieldBase(rest),
      value,
      min,
      max,
      step,
      type: "number" /* NUMBER */,
  });
  const SelectField = ({ options = [], value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      options,
      type: "select" /* SELECT */,
  });
  const CustomField = ({ value, ...rest }) => ({
      ...FieldBase(rest),
      value,
      type: "custom-field" /* CUSTOM */,
  });

  const isEmptyInputValue = (value) => value == null || value === '';
  const EMAIL_REGEXP = /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/;
  const URL_REGEXP = /^((?:(https?):\/\/)?((?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9]))|(?:(?:(?:\w+\.){1,2}[\w]{2,3})))(?::(\d+))?((?:\/[\w]+)*)(?:\/|(\/[\w]+\.[\w]{3,4})|(\?(?:([\w]+=[\w]+)&)*([\w]+=[\w]+))?|\?(?:(wsdl|wadl))))$/;
  const required = (control) => isEmptyInputValue(control.value) ? { required: true } : null;
  const min = (min) => (control) => {
      if (isEmptyInputValue(control.value) || isEmptyInputValue(min)) {
          return null; // don't validate empty values to allow optional controls
      }
      const value = parseFloat(`${control.value}`);
      return !isNaN(value) && value < min
          ? { min: { min, actual: +control.value } }
          : null;
  };
  const max = (max) => (control) => {
      if (isEmptyInputValue(control.value) || isEmptyInputValue(max)) {
          return null; // don't validate empty values to allow optional controls
      }
      const value = parseFloat(`${control.value}`);
      // Controls with NaN values after parsing should be treated as not having a
      // maximum, per the HTML forms spec: https://www.w3.org/TR/html5/forms.html#attr-input-max
      return !isNaN(value) && value > max
          ? { max: { max, actual: control.value } }
          : null;
  };
  const email = (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      return EMAIL_REGEXP.test(`${control.value}`) ? null : { email: true };
  };
  const url = (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      return URL_REGEXP.test(`${control.value}`) ? null : { email: true };
  };
  const minLength = (minLength) => (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      const length = control.value ? `${control.value}`.length : 0;
      return length < minLength
          ? { minlength: { requiredLength: minLength, actualLength: length } }
          : null;
  };
  const maxLength = (maxLength) => (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      const length = control.value ? `${control.value}`.length : 0;
      return length > maxLength
          ? { maxlength: { requiredLength: maxLength, actualLength: length } }
          : null;
  };
  const pattern = (pattern) => (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      let regexStr;
      regexStr = '';
      if (pattern.charAt(0) !== '^')
          regexStr += '^';
      regexStr += pattern;
      if (pattern.charAt(pattern.length - 1) !== '$')
          regexStr += '$';
      const regex = new RegExp(regexStr);
      const value = `${control.value}`;
      return regex.test(value)
          ? { pattern: { requiredPattern: regexStr, actualValue: value } }
          : null;
  };

  exports.CheckboxField = CheckboxField;
  exports.ColorField = ColorField;
  exports.CustomField = CustomField;
  exports.EmailField = EmailField;
  exports.FieldBase = FieldBase;
  exports.NumberField = NumberField;
  exports.PasswordField = PasswordField;
  exports.RadioField = RadioField;
  exports.SelectField = SelectField;
  exports.TextField = TextField;
  exports.createDynamicForms = createDynamicForms;
  exports.dynamicFormsSymbol = dynamicFormsSymbol;
  exports.email = email;
  exports.isEmptyInputValue = isEmptyInputValue;
  exports.max = max;
  exports.maxLength = maxLength;
  exports.min = min;
  exports.minLength = minLength;
  exports.pattern = pattern;
  exports.required = required;
  exports.url = url;
  exports.useDynamicForms = useDynamicForms;

  Object.defineProperty(exports, '__esModule', { value: true });

  return exports;

}({}, Vue));
